@@ -64,6 +64,7 @@ struct db_breakpoint {
 #define BKPT_USR_GLOBAL		0x8	/* global user space break point */
 #define BKPT_SET_IN_MEM		0x10	/* break point is set in memory */
 #define BKPT_1ST_SET		0x20	/* 1st time set of user global bkpt */
+#define BKPT_EXTERNAL		0x40	/* break point is not from ddb */
 	vm_size_t	bkpt_inst;	/* saved instruction at bkpt */
 	struct db_breakpoint *link;	/* link in in-use or free chain */
 };
@@ -72,8 +73,8 @@ typedef struct db_breakpoint *db_breakpoint_t;
 
 extern db_breakpoint_t	db_find_breakpoint( task_t task, db_addr_t addr);
 extern boolean_t	db_find_breakpoint_here( task_t task, db_addr_t addr);
-extern void		db_set_breakpoints();
-extern void		db_clear_breakpoints();
+extern void		db_set_breakpoints(void);
+extern void		db_clear_breakpoints(void);
 extern db_thread_breakpoint_t	db_find_thread_breakpoint_here
 					( task_t task, db_addr_t addr );
 extern db_thread_breakpoint_t	db_find_breakpoint_number
@@ -83,4 +84,18 @@ extern db_breakpoint_t	db_set_temp_breakpoint( task_t task, db_addr_t addr);
 extern void		db_delete_temp_breakpoint
 					( task_t task, db_breakpoint_t bkpt);
 
+extern db_breakpoint_t  db_set_breakpoint(task_t task, db_addr_t addr, 
+					  int count, thread_t thread, 
+					  boolean_t task_bpt);
+
+void db_listbreak_cmd();
+
+void db_delete_cmd();
+
+void db_breakpoint_cmd(
+	db_expr_t	addr,
+	int		have_addr,
+	db_expr_t	count,
+	char *		modif);
+
 #endif	/* _DDB_DB_BREAK_H_ */
