@@ -24,11 +24,8 @@
  * the rights to redistribute these changes.
  */
 
-#include "mach_kdb.h"
 #if MACH_KDB
 
-#include <mach_debug.h>
-
 #if	MACH_DEBUG
 
 #include <mach/mach_types.h>	/* vm_address_t */
@@ -36,6 +33,7 @@
 #include <mach/vm_param.h>
 #include <vm/vm_map.h>
 #include <vm/vm_kern.h>
+#include <vm/vm_user.h>
 #include <kern/host.h>
 #include <kern/task.h>
 #include <ddb/db_sym.h>
