@@ -32,9 +32,9 @@
  * Printing routines for kernel debugger.
  */
 
-extern void	db_force_whitespace();
-extern int	db_print_position();
-extern void	db_end_line();
-extern void	db_printf( const char *fmt, ...);
-
-
+extern void	db_force_whitespace(void);
+extern int	db_print_position(void);
+extern void	db_end_line(void);
+extern void	db_printf(const char *fmt, ...);
+extern void	db_putchar(int c);
+extern void	kdbprintf(const char *fmt, ...);
