@@ -35,11 +35,14 @@
  */
 #include <mach/boolean.h>
 #include <machine/db_machdep.h>
+#include <machine/setjmp.h>
 #include <ddb/db_command.h>
 #include <ddb/db_access.h>
 #include <ddb/db_break.h>
+#include <ddb/db_examine.h>
+#include <ddb/db_output.h>
 #include <ddb/db_task_thread.h>
-
+#include <ddb/db_trap.h>
 
 
 extern jmp_buf_t *db_recover;
