@@ -33,17 +33,17 @@
 #include <ipc/ipc_space.h>
 #include <kern/debug.h>
 #include <kern/task.h>
+#include <xen/xen.h>
 
 #include <device/device_types.h>
 #include <device/device_port.h>
+#include <device/tty.h>
 
 
-
-extern void	ds_init();
+extern void	mach_device_init();
 extern void	dev_lookup_init();
 extern void	net_io_init();
 extern void	device_pager_init();
-extern void	chario_init(void);
 
 extern void	io_done_thread();
 extern void	net_thread();
@@ -57,7 +57,10 @@ device_service_create()
 	if (master_device_port == IP_NULL)
 	    panic("can't allocate master device port");
 
-	ds_init();
+	mach_device_init();
+#ifdef MACH_HYP
+	hyp_dev_init();
+#endif
 	dev_lookup_init();
 	net_io_init();
 	device_pager_init();
