@@ -35,18 +35,34 @@
 
 #include <vm/vm_map.h>
 #include <device/device_types.h>
+#include <device/io_req.h>
+#include <mach/machine/vm_types.h>
 
 /*
  * Map for device IO memory.
  */
-vm_map_t	device_io_map;
+extern vm_map_t		device_io_map;
 
-kern_return_t	device_read_alloc();
-kern_return_t	device_write_get();
-boolean_t	device_write_dealloc();
+extern queue_head_t	io_done_list;
 
-boolean_t	ds_open_done();
-boolean_t	ds_read_done();
-boolean_t	ds_write_done();
+kern_return_t	device_read_alloc(io_req_t, vm_size_t);
+kern_return_t	device_write_get(io_req_t, boolean_t *);
+boolean_t	device_write_dealloc(io_req_t);
+void		device_reference(device_t);
+void		device_deallocate(device_t);
+
+boolean_t	ds_notify(mach_msg_header_t *msg);
+boolean_t	ds_open_done(io_req_t);
+boolean_t	ds_read_done(io_req_t);
+boolean_t	ds_write_done(io_req_t);
+
+void		iowait (io_req_t ior);
+
+kern_return_t	device_pager_setup(
+	mach_device_t	device,
+	int		prot,
+	vm_offset_t	offset,
+	vm_size_t	size,
+	mach_port_t	*pager);
 
 #endif	/* DS_ROUTINES_H */
