@@ -42,6 +42,8 @@
 #include <kern/lock.h>
 #include <kern/kalloc.h>
 
+#include <device/if_hdr.h>
+#include <device/io_req.h>
 #include <device/net_status.h>
 
 /*
@@ -56,17 +58,19 @@
  * functions.  net_kmsg_get may return IKM_NULL.
  */
 
-extern ipc_kmsg_t net_kmsg_get();
-extern void net_kmsg_put();
+extern ipc_kmsg_t net_kmsg_get(void);
+extern void net_kmsg_put(ipc_kmsg_t);
 
 /*
  * Network utility routines.
  */
 
-extern void net_packet();
-extern void net_filter();
-extern io_return_t net_getstat();
-extern io_return_t net_write();
+extern void net_ast();
+extern void net_packet(struct ifnet *, ipc_kmsg_t, unsigned int, boolean_t);
+extern void net_filter(ipc_kmsg_t, ipc_kmsg_queue_t);
+extern io_return_t net_getstat(struct ifnet *, dev_flavor_t, dev_status_t,
+			       natural_t *);
+extern io_return_t net_write(struct ifnet *, int (*)(), io_req_t);
 
 /*
  * Non-interrupt code may allocate and free net_kmsgs with these functions.
@@ -74,7 +78,14 @@ extern io_return_t net_write();
 
 extern vm_size_t net_kmsg_size;
 
+extern void net_kmsg_collect (void);
+
 #define net_kmsg_alloc()	((ipc_kmsg_t) kalloc(net_kmsg_size))
 #define net_kmsg_free(kmsg)	kfree((vm_offset_t) (kmsg), net_kmsg_size)
 
+extern unsigned int		ntohl(unsigned int);
+extern unsigned short int	ntohs(unsigned short int);
+extern unsigned int		htonl(unsigned int);
+extern unsigned short int	htons(unsigned short int);
+
 #endif	/* _DEVICE_NET_IO_H_ */
