@@ -23,6 +23,15 @@
  * any improvements or extensions that they make and grant Carnegie Mellon
  * the rights to redistribute these changes.
  */
+
+/*
+ *  Copyright (C) 1994 Linus Torvalds
+ *
+ *  Pentium III FXSR, SSE support
+ *  General FPU state handling cleanups
+ *	Gareth Hughes <gareth@valinux.com>, May 2000
+ */
+
 /*
  * Support for 80387 floating point or FP emulator.
  */
@@ -35,19 +44,19 @@
 
 #include <kern/debug.h>
 #include <machine/machspl.h>	/* spls */
-#include <kern/mach_param.h>
 #include <kern/printf.h>
 #include <kern/thread.h>
-#include <kern/zalloc.h>
+#include <kern/slab.h>
 
 #include <i386/thread.h>
 #include <i386/fpu.h>
 #include <i386/pio.h>
+#include <i386/pic.h>
+#include <i386/locore.h>
 #include "cpu_number.h"
 
 #if 0
 #include <i386/ipl.h>
-extern int curr_ipl;
 #define ASSERT_IPL(L) \
 { \
       if (curr_ipl != L) { \
@@ -62,7 +71,11 @@ extern int curr_ipl;
 extern void i386_exception();
 
 int		fp_kind = FP_387;	/* 80387 present */
-zone_t		ifps_zone;		/* zone for FPU save area */
+struct kmem_cache	ifps_cache;	/* cache for FPU save area */
+static unsigned long	mxcsr_feature_mask = 0xffffffff;	/* Always AND user-provided mxcsr with this security mask */
+
+void fp_save(thread_t thread);
+void fp_load(thread_t thread);
 
 #if	NCPUS == 1
 volatile thread_t	fp_thread = THREAD_NULL;
@@ -96,12 +109,21 @@ init_fpu()
 {
 	unsigned short	status, control;
 
+#ifdef	MACH_RING1
+	clear_ts();
+#else	/* MACH_RING1 */
+	unsigned int native = 0;
+
+	if (machine_slot[cpu_number()].cpu_type >= CPU_TYPE_I486)
+		native = CR0_NE;
+
 	/*
 	 * Check for FPU by initializing it,
 	 * then trying to read the correct bit patterns from
 	 * the control and status registers.
 	 */
-	set_cr0(get_cr0() & ~(CR0_EM|CR0_TS));	/* allow use of FPU */
+	set_cr0((get_cr0() & ~(CR0_EM|CR0_TS)) | native);	/* allow use of FPU */
+#endif	/* MACH_RING1 */
 
 	fninit();
 	status = fnstsw();
@@ -130,27 +152,36 @@ init_fpu()
 		/*
 		 * We have a 387.
 		 */
-		fp_kind = FP_387;
+		if (CPU_HAS_FEATURE(CPU_FEATURE_FXSR)) {
+		    static /* because we _need_ alignment */
+		    struct i386_xfp_save save;
+		    unsigned long mask;
+		    fp_kind = FP_387X;
+#ifndef MACH_RING1
+		    set_cr4(get_cr4() | CR4_OSFXSR);
+#endif /* MACH_RING1 */
+		    fxsave(&save);
+		    mask = save.fp_mxcsr_mask;
+		    if (!mask)
+			mask = 0x0000ffbf;
+		    mxcsr_feature_mask &= mask;
+		} else
+		    fp_kind = FP_387;
 	    }
+#ifdef MACH_RING1
+	    set_ts();
+#else	/* MACH_RING1 */
 	    /*
 	     * Trap wait instructions.  Turn off FPU for now.
 	     */
 	    set_cr0(get_cr0() | CR0_TS | CR0_MP);
+#endif	/* MACH_RING1 */
 	}
 	else {
-#if	FPE
-	    /*
-	     * Use the floating-point emulator.
-	     */
-	    fp_kind = FP_SOFT;
-	    fpe_init();
-#else	/* no fpe */
 	    /*
 	     * NO FPU.
 	     */
-	    fp_kind = FP_NO;
-	    set_cr0(get_cr0() | CR0_EM);
-#endif
+	    panic("No FPU!");
 	}
 }
 
@@ -160,10 +191,9 @@ init_fpu()
 void
 fpu_module_init()
 {
-	ifps_zone = zinit(sizeof(struct i386_fpsave_state),
-			  THREAD_MAX * sizeof(struct i386_fpsave_state),
-			  THREAD_CHUNK * sizeof(struct i386_fpsave_state),
-			  0, "i386 fpsave state");
+	kmem_cache_init(&ifps_cache, "i386_fpsave_state",
+			sizeof(struct i386_fpsave_state), 16,
+			NULL, NULL, NULL, 0);
 }
 
 /*
@@ -188,7 +218,74 @@ ASSERT_IPL(SPL0);
 	    clear_fpu();
 	}
 #endif	/* NCPUS == 1 */
-	zfree(ifps_zone, (vm_offset_t) fps);
+	kmem_cache_free(&ifps_cache, (vm_offset_t) fps);
+}
+
+/* The two following functions were stolen from Linux's i387.c */
+static inline unsigned short
+twd_i387_to_fxsr (unsigned short twd)
+{
+	unsigned int tmp; /* to avoid 16 bit prefixes in the code */
+
+	/* Transform each pair of bits into 01 (valid) or 00 (empty) */
+	tmp = ~twd;
+	tmp = (tmp | (tmp>>1)) & 0x5555; /* 0V0V0V0V0V0V0V0V */
+	/* and move the valid bits to the lower byte. */
+	tmp = (tmp | (tmp >> 1)) & 0x3333; /* 00VV00VV00VV00VV */
+	tmp = (tmp | (tmp >> 2)) & 0x0f0f; /* 0000VVVV0000VVVV */
+	tmp = (tmp | (tmp >> 4)) & 0x00ff; /* 00000000VVVVVVVV */
+	return tmp;
+}
+
+static inline unsigned long
+twd_fxsr_to_i387 (struct i386_xfp_save *fxsave)
+{
+	struct {
+		unsigned short significand[4];
+		unsigned short exponent;
+		unsigned short padding[3];
+	} *st = NULL;
+	unsigned long tos = (fxsave->fp_status >> 11) & 7;
+	unsigned long twd = (unsigned long) fxsave->fp_tag;
+	unsigned long tag;
+	unsigned long ret = 0xffff0000u;
+	int i;
+
+#define FPREG_ADDR(f, n)	((void *)&(f)->fp_reg_word + (n) * 16);
+
+	for (i = 0 ; i < 8 ; i++) {
+		if (twd & 0x1) {
+			st = FPREG_ADDR (fxsave, (i - tos) & 7);
+
+			switch (st->exponent & 0x7fff) {
+			case 0x7fff:
+				tag = 2;		 /* Special */
+				break;
+			case 0x0000:
+				if (!st->significand[0] &&
+				    !st->significand[1] &&
+				    !st->significand[2] &&
+				    !st->significand[3] ) {
+					tag = 1;	/* Zero */
+				} else {
+					tag = 2;	/* Special */
+				}
+				break;
+			 default:
+				if (st->significand[3] & 0x8000) {
+					tag = 0;	/* Valid */
+				} else {
+					tag = 2;	/* Special */
+				}
+				break;
+			}
+		} else {
+			tag = 3;			/* Empty */
+		}
+		ret |= (tag << (2 * i));
+		twd = twd >> 1;
+	}
+	return ret;
 }
 
 /*
@@ -235,7 +332,7 @@ ASSERT_IPL(SPL0);
 	    simple_unlock(&pcb->lock);
 
 	    if (ifps != 0) {
-		zfree(ifps_zone, (vm_offset_t) ifps);
+		kmem_cache_free(&ifps_cache, (vm_offset_t) ifps);
 	    }
 	}
 	else {
@@ -256,7 +353,7 @@ ASSERT_IPL(SPL0);
 	    if (ifps == 0) {
 		if (new_ifps == 0) {
 		    simple_unlock(&pcb->lock);
-		    new_ifps = (struct i386_fpsave_state *) zalloc(ifps_zone);
+		    new_ifps = (struct i386_fpsave_state *) kmem_cache_alloc(&ifps_cache);
 		    goto Retry;
 		}
 		ifps = new_ifps;
@@ -269,51 +366,34 @@ ASSERT_IPL(SPL0);
 	     */
 	    memset(&ifps->fp_save_state, 0, sizeof(struct i386_fp_save));
 
-	    ifps->fp_save_state.fp_control = user_fp_state->fp_control;
-	    ifps->fp_save_state.fp_status  = user_fp_state->fp_status;
-	    ifps->fp_save_state.fp_tag     = user_fp_state->fp_tag;
-	    ifps->fp_save_state.fp_eip     = user_fp_state->fp_eip;
-	    ifps->fp_save_state.fp_cs      = user_fp_state->fp_cs;
-	    ifps->fp_save_state.fp_opcode  = user_fp_state->fp_opcode;
-	    ifps->fp_save_state.fp_dp      = user_fp_state->fp_dp;
-	    ifps->fp_save_state.fp_ds      = user_fp_state->fp_ds;
+	    if (fp_kind == FP_387X) {
+		int i;
 
-#if	FPE
-	    if (fp_kind == FP_SOFT) {
-		/*
-		 * The emulator stores the registers by physical
-		 * register number, not from top-of-stack.
-		 * Shuffle the registers into the correct order.
-		 */
-		register char *src;	/* user regs */
-		register char *dst;	/* kernel regs */
-		int	i;
-
-		src = (char *)user_fp_regs;
-		dst = (char *)&ifps->fp_regs;
-		i = (ifps->fp_save_state.fp_status & FPS_TOS)
-			>> FPS_TOS_SHIFT;	/* physical register
-						   for st(0) */
-		if (i == 0)
-		    memcpy(dst, src, 8 * 10);
-		else {
-		    memcpy(dst + 10 * i,
-			  src, 
-			  10 * (8 - i));
-		    memcpy(dst, 
-			  src + 10 * (8 - i),
-			  10 * i);
-		}
-	    }
-	    else
+		ifps->xfp_save_state.fp_control = user_fp_state->fp_control;
+		ifps->xfp_save_state.fp_status  = user_fp_state->fp_status;
+		ifps->xfp_save_state.fp_tag     = twd_i387_to_fxsr(user_fp_state->fp_tag);
+		ifps->xfp_save_state.fp_eip     = user_fp_state->fp_eip;
+		ifps->xfp_save_state.fp_cs      = user_fp_state->fp_cs;
+		ifps->xfp_save_state.fp_opcode  = user_fp_state->fp_opcode;
+		ifps->xfp_save_state.fp_dp      = user_fp_state->fp_dp;
+		ifps->xfp_save_state.fp_ds      = user_fp_state->fp_ds;
+		for (i=0; i<8; i++)
+		    memcpy(&ifps->xfp_save_state.fp_reg_word[i], &user_fp_regs[i], sizeof(user_fp_regs[i]));
+	    } else {
+		ifps->fp_save_state.fp_control = user_fp_state->fp_control;
+		ifps->fp_save_state.fp_status  = user_fp_state->fp_status;
+		ifps->fp_save_state.fp_tag     = user_fp_state->fp_tag;
+		ifps->fp_save_state.fp_eip     = user_fp_state->fp_eip;
+		ifps->fp_save_state.fp_cs      = user_fp_state->fp_cs;
+		ifps->fp_save_state.fp_opcode  = user_fp_state->fp_opcode;
+		ifps->fp_save_state.fp_dp      = user_fp_state->fp_dp;
+		ifps->fp_save_state.fp_ds      = user_fp_state->fp_ds;
 		ifps->fp_regs = *user_fp_regs;
-#else	/* no FPE */
-	    ifps->fp_regs = *user_fp_regs;
-#endif	/* FPE */
+	    }
 
 	    simple_unlock(&pcb->lock);
 	    if (new_ifps != 0)
-		zfree(ifps_zone, (vm_offset_t) ifps);
+		kmem_cache_free(&ifps_cache, (vm_offset_t) new_ifps);
 	}
 
 	return KERN_SUCCESS;
@@ -379,47 +459,30 @@ ASSERT_IPL(SPL0);
 	     */
 	    memset(user_fp_state,  0, sizeof(struct i386_fp_save));
 
-	    user_fp_state->fp_control = ifps->fp_save_state.fp_control;
-	    user_fp_state->fp_status  = ifps->fp_save_state.fp_status;
-	    user_fp_state->fp_tag     = ifps->fp_save_state.fp_tag;
-	    user_fp_state->fp_eip     = ifps->fp_save_state.fp_eip;
-	    user_fp_state->fp_cs      = ifps->fp_save_state.fp_cs;
-	    user_fp_state->fp_opcode  = ifps->fp_save_state.fp_opcode;
-	    user_fp_state->fp_dp      = ifps->fp_save_state.fp_dp;
-	    user_fp_state->fp_ds      = ifps->fp_save_state.fp_ds;
+	    if (fp_kind == FP_387X) {
+		int i;
 
-#if	FPE
-	    if (fp_kind == FP_SOFT) {
-		/*
-		 * The emulator stores the registers by physical
-		 * register number, not from top-of-stack.
-		 * Shuffle the registers into the correct order.
-		 */
-		register char *src;	/* kernel regs */
-		register char *dst;	/* user regs */
-		int	i;
-
-		src = (char *)&ifps->fp_regs;
-		dst = (char *)user_fp_regs;
-		i = (ifps->fp_save_state.fp_status & FPS_TOS)
-			>> FPS_TOS_SHIFT;	/* physical register
-						   for st(0) */
-		if (i == 0)
-		    memcpy(dst, src, 8 * 10);
-		else {
-		    memcpy(dst, 
-			  src + 10 * i,
-			  10 * (8 - i));
-		    memcpy(dst + 10 * (8 - i), 
-			  src, 
-			  10 * i);
-		}
-	    }
-	    else
+		user_fp_state->fp_control = ifps->xfp_save_state.fp_control;
+		user_fp_state->fp_status  = ifps->xfp_save_state.fp_status;
+		user_fp_state->fp_tag     = twd_fxsr_to_i387(&ifps->xfp_save_state);
+		user_fp_state->fp_eip     = ifps->xfp_save_state.fp_eip;
+		user_fp_state->fp_cs      = ifps->xfp_save_state.fp_cs;
+		user_fp_state->fp_opcode  = ifps->xfp_save_state.fp_opcode;
+		user_fp_state->fp_dp      = ifps->xfp_save_state.fp_dp;
+		user_fp_state->fp_ds      = ifps->xfp_save_state.fp_ds;
+		for (i=0; i<8; i++)
+		    memcpy(&user_fp_regs[i], &ifps->xfp_save_state.fp_reg_word[i], sizeof(user_fp_regs[i]));
+	    } else {
+		user_fp_state->fp_control = ifps->fp_save_state.fp_control;
+		user_fp_state->fp_status  = ifps->fp_save_state.fp_status;
+		user_fp_state->fp_tag     = ifps->fp_save_state.fp_tag;
+		user_fp_state->fp_eip     = ifps->fp_save_state.fp_eip;
+		user_fp_state->fp_cs      = ifps->fp_save_state.fp_cs;
+		user_fp_state->fp_opcode  = ifps->fp_save_state.fp_opcode;
+		user_fp_state->fp_dp      = ifps->fp_save_state.fp_dp;
+		user_fp_state->fp_ds      = ifps->fp_save_state.fp_ds;
 		*user_fp_regs = ifps->fp_regs;
-#else	/* no FPE */
-	    *user_fp_regs = ifps->fp_regs;
-#endif	/* FPE */
+	    }
 	}
 	simple_unlock(&pcb->lock);
 
@@ -543,7 +606,7 @@ fpextovrflt()
 	clear_fpu();
 
 	if (ifps)
-	    zfree(ifps_zone, (vm_offset_t) ifps);
+	    kmem_cache_free(&ifps_cache, (vm_offset_t) ifps);
 
 	/*
 	 * Raise exception.
@@ -552,11 +615,80 @@ fpextovrflt()
 	/*NOTREACHED*/
 }
 
+static int
+fphandleerr()
+{
+	register thread_t	thread = current_thread();
+
+	/*
+	 * Save the FPU context to the thread using it.
+	 */
+#if	NCPUS == 1
+	if (fp_thread == THREAD_NULL) {
+		printf("fphandleerr: FPU not belonging to anyone!\n");
+		clear_ts();
+		fninit();
+		clear_fpu();
+		return 1;
+	}
+
+	if (fp_thread != thread) {
+	    /*
+	     * FPU exception is for a different thread.
+	     * When that thread again uses the FPU an exception will be
+	     * raised in fp_load. Remember the condition in fp_valid (== 2).
+	     */
+	    clear_ts();
+	    fp_save(fp_thread);
+	    fp_thread->pcb->ims.ifps->fp_valid = 2;
+	    fninit();
+	    clear_fpu();
+	    /* leave fp_intr_thread THREAD_NULL */
+	    return 1;
+	}
+#endif	/* NCPUS == 1 */
+
+	/*
+	 * Save the FPU state and turn off the FPU.
+	 */
+	clear_ts();
+	fp_save(thread);
+	fninit();
+	clear_fpu();
+
+	return 0;
+}
+
+/*
+ * FPU error. Called by exception handler.
+ */
+void
+fpexterrflt()
+{
+	register thread_t	thread = current_thread();
+
+	if (fphandleerr())
+		return;
+
+	/*
+	 * Raise FPU exception.
+	 * Locking not needed on pcb->ims.ifps,
+	 * since thread is running.
+	 */
+	i386_exception(EXC_ARITHMETIC,
+		       EXC_I386_EXTERR,
+		       fp_kind == FP_387X ?
+		           thread->pcb->ims.ifps->xfp_save_state.fp_status :
+		           thread->pcb->ims.ifps->fp_save_state.fp_status);
+	/*NOTREACHED*/
+}
+
+#ifndef MACH_RING1
 /*
  * FPU error. Called by AST.
  */
 void
-fpexterrflt()
+fpastintr()
 {
 	register thread_t	thread = current_thread();
 
@@ -603,9 +735,12 @@ ASSERT_IPL(SPL0);
 	 */
 	i386_exception(EXC_ARITHMETIC,
 		       EXC_I386_EXTERR,
-		       thread->pcb->ims.ifps->fp_save_state.fp_status);
+		       fp_kind == FP_387X ?
+		           thread->pcb->ims.ifps->xfp_save_state.fp_status :
+		           thread->pcb->ims.ifps->fp_save_state.fp_status);
 	/*NOTREACHED*/
 }
+#endif /* MACH_RING1 */
 
 /*
  * Save FPU state.
@@ -625,7 +760,10 @@ fp_save(thread)
 	if (ifps != 0 && !ifps->fp_valid) {
 	    /* registers are in FPU */
 	    ifps->fp_valid = TRUE;
-	    fnsave(&ifps->fp_save_state);
+	    if (fp_kind == FP_387X)
+	    	fxsave(&ifps->xfp_save_state);
+	    else
+	    	fnsave(&ifps->fp_save_state);
 	}
 }
 
@@ -644,7 +782,7 @@ fp_load(thread)
 ASSERT_IPL(SPL0);
 	ifps = pcb->ims.ifps;
 	if (ifps == 0) {
-	    ifps = (struct i386_fpsave_state *) zalloc(ifps_zone);
+	    ifps = (struct i386_fpsave_state *) kmem_cache_alloc(&ifps_cache);
 	    memset(ifps, 0, sizeof *ifps);
 	    pcb->ims.ifps = ifps;
 	    fpinit();
@@ -666,14 +804,19 @@ ASSERT_IPL(SPL0);
 		 */
 		i386_exception(EXC_ARITHMETIC,
 			       EXC_I386_EXTERR,
-			       thread->pcb->ims.ifps->fp_save_state.fp_status);
+			       fp_kind == FP_387X ?
+			           thread->pcb->ims.ifps->xfp_save_state.fp_status :
+			           thread->pcb->ims.ifps->fp_save_state.fp_status);
 		/*NOTREACHED*/
 #endif
 	} else if (! ifps->fp_valid) {
 		printf("fp_load: invalid FPU state!\n");
 		fninit ();
 	} else {
-	    frstor(ifps->fp_save_state);
+	    if (fp_kind == FP_387X)
+		fxrstor(ifps->xfp_save_state);
+	    else
+		frstor(ifps->fp_save_state);
 	}
 	ifps->fp_valid = FALSE;		/* in FPU */
 }
@@ -690,25 +833,36 @@ fp_state_alloc()
 	pcb_t	pcb = current_thread()->pcb;
 	struct i386_fpsave_state *ifps;
 
-	ifps = (struct i386_fpsave_state *)zalloc(ifps_zone);
+	ifps = (struct i386_fpsave_state *)kmem_cache_alloc(&ifps_cache);
 	memset(ifps, 0, sizeof *ifps);
 	pcb->ims.ifps = ifps;
 
 	ifps->fp_valid = TRUE;
-	ifps->fp_save_state.fp_control = (0x037f
-			& ~(FPC_IM|FPC_ZM|FPC_OM|FPC_PC))
-			| (FPC_PC_53|FPC_IC_AFF);
-	ifps->fp_save_state.fp_status = 0;
-	ifps->fp_save_state.fp_tag = 0xffff;	/* all empty */
+
+	if (fp_kind == FP_387X) {
+		ifps->xfp_save_state.fp_control = (0x037f
+				& ~(FPC_IM|FPC_ZM|FPC_OM|FPC_PC))
+				| (FPC_PC_53|FPC_IC_AFF);
+		ifps->xfp_save_state.fp_status = 0;
+		ifps->xfp_save_state.fp_tag = 0xffff;	/* all empty */
+		if (CPU_HAS_FEATURE(CPU_FEATURE_SSE))
+			ifps->xfp_save_state.fp_mxcsr = 0x1f80;
+	} else {
+		ifps->fp_save_state.fp_control = (0x037f
+				& ~(FPC_IM|FPC_ZM|FPC_OM|FPC_PC))
+				| (FPC_PC_53|FPC_IC_AFF);
+		ifps->fp_save_state.fp_status = 0;
+		ifps->fp_save_state.fp_tag = 0xffff;	/* all empty */
+	}
 }
 
-#if	AT386
+#if	AT386 && !defined(MACH_XEN)
 /*
  *	Handle a coprocessor error interrupt on the AT386.
  *	This comes in on line 5 of the slave PIC at SPL1.
  */
 void
-fpintr()
+fpintr(int unit)
 {
 	spl_t	s;
 	thread_t thread = current_thread();
@@ -719,42 +873,15 @@ ASSERT_IPL(SPL1);
 	 */
 	outb(0xf0, 0);
 
-	/*
-	 * Save the FPU context to the thread using it.
-	 */
-#if	NCPUS == 1
-	if (fp_thread == THREAD_NULL) {
-		printf("fpintr: FPU not belonging to anyone!\n");
-		clear_ts();
-		fninit();
-		clear_fpu();
+	if (fphandleerr())
 		return;
-	}
 
-	if (fp_thread != thread) {
-	    /*
-	     * FPU exception is for a different thread.
-	     * When that thread again uses the FPU an exception will be
-	     * raised in fp_load. Remember the condition in fp_valid (== 2).
-	     */
-	    clear_ts();
-	    fp_save(fp_thread);
-	    fp_thread->pcb->ims.ifps->fp_valid = 2;
-	    fninit();
-	    clear_fpu();
-	    /* leave fp_intr_thread THREAD_NULL */
-	    return;
-	}
-	if (fp_intr_thread != THREAD_NULL)
+#if	NCPUS == 1
+	if (fp_intr_thread != THREAD_NULL && fp_intr_thread != thread)
 	    panic("fp_intr: already caught intr");
 	fp_intr_thread = thread;
 #endif	/* NCPUS == 1 */
 
-	clear_ts();
-	fp_save(thread);
-	fninit();
-	clear_fpu();
-
 	/*
 	 * Since we are running on the interrupt stack, we must
 	 * signal the thread to take the exception when we return
