@@ -49,12 +49,12 @@ OTHER TORTIOUS ACTION, ARISING OUR OF OR IN CONNECTION
 WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <platforms.h>
-
 #include <sys/types.h>
+#include <kern/printf.h>
 #include <i386/ipl.h>
 #include <i386/pic.h>
 #include <i386/machspl.h>
+#include <i386/pio.h>
 
 spl_t	curr_ipl;
 int	pic_mask[NSPL];
@@ -65,7 +65,7 @@ int	iunit[NINTR] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
 int 	nintr = NINTR;
 int	npics = NPICS;
 
-char	*master_icw, *master_ocw, *slaves_icw, *slaves_ocw;
+unsigned short	master_icw, master_ocw, slaves_icw, slaves_ocw;
 
 u_short PICM_ICW1, PICM_OCW1, PICS_ICW1, PICS_OCW1 ;
 u_short PICM_ICW2, PICM_OCW2, PICS_ICW2, PICS_OCW2 ;
@@ -104,11 +104,10 @@ u_short PICM_ICW4, PICS_ICW4 ;
 **
 */
 
+void
 picinit()
 {
 
-	u_short i;
-
 	asm("cli");
 
 	/*
@@ -131,13 +130,11 @@ picinit()
 	** 2. Generate addresses to each PIC port.
 	*/
 
-	master_icw = (char *)PIC_MASTER_ICW;
-	master_ocw = (char *)PIC_MASTER_OCW;
-	slaves_icw = (char *)PIC_SLAVE_ICW;
-	slaves_ocw = (char *)PIC_SLAVE_OCW;
+	master_icw = PIC_MASTER_ICW;
+	master_ocw = PIC_MASTER_OCW;
+	slaves_icw = PIC_SLAVE_ICW;
+	slaves_ocw = PIC_SLAVE_OCW;
 
-#ifdef	PS2
-#else	/* PS2 */
 	/*
 	** 3. Select options for each ICW and each OCW for each PIC.
 	*/
@@ -155,23 +152,11 @@ picinit()
 	PICM_ICW3 = ( SLAVE_ON_IR2 );
 	PICS_ICW3 = ( I_AM_SLAVE_2 );
 #endif	/* AT386 */
-#ifdef	iPSC386
-        PICM_ICW3 = ( SLAVE_ON_IR7 );
-        PICS_ICW3 = ( I_AM_SLAVE_7 );
-#endif	/* iPSC386 */
 
-#ifdef	iPSC386
-        /* Use Buffered mode for iPSC386 */
-        PICM_ICW4 = (SNF_MODE_DIS | BUFFERD_MODE | I_AM_A_MASTR |
-                     NRML_EOI_MOD | I8086_EMM_MOD);
-        PICS_ICW4 = (SNF_MODE_DIS | BUFFERD_MODE | I_AM_A_SLAVE |
-                     NRML_EOI_MOD | I8086_EMM_MOD);
-#else	/* iPSC386 */
 	PICM_ICW4 =
  	(SNF_MODE_DIS | NONBUFD_MODE | NRML_EOI_MOD | I8086_EMM_MOD);
 	PICS_ICW4 =
  	(SNF_MODE_DIS | NONBUFD_MODE | NRML_EOI_MOD | I8086_EMM_MOD);
-#endif	/* iPSC386 */
 
 	PICM_OCW1 = (curr_pic_mask & 0x00FF);
 	PICS_OCW1 = ((curr_pic_mask & 0xFF00)>>8);
@@ -213,8 +198,6 @@ picinit()
 	*/
 	outb ( master_ocw, PICM_OCW1 );
 
-#endif	/* PS2 */
-
 #if 0
 	printf(" spl set to %x \n", curr_pic_mask);
 #endif
@@ -233,15 +216,13 @@ picinit()
 **
 */
 
-#if	defined(AT386) || defined(PS2)
+#if	defined(AT386)
 #define SLAVEMASK       (0xFFFF ^ SLAVE_ON_IR2)
-#endif	/* defined(AT386) || defined(PS2) */
-#ifdef	iPSC386
-#define SLAVEMASK       (0xFFFF ^ SLAVE_ON_IR7)
-#endif	/* iPSC386 */
+#endif	/* defined(AT386) */
 
 #define SLAVEACTV	0xFF00
 
+void
 form_pic_mask()
 {
 	unsigned int i, j, bit, mask;
@@ -258,12 +239,15 @@ form_pic_mask()
 	}
 }
 
+void
 intnull(unit_dev)
 {
 	printf("intnull(%d)\n", unit_dev);
 }
 
 int prtnull_count = 0;
+
+void
 prtnull(unit)
 {
 	++prtnull_count;
