@@ -30,12 +30,11 @@
 #include <mach/machine/vm_types.h>
 #include <device/conf.h>
 
-extern vm_offset_t block_io_mmap();
-
 extern int	timeopen(), timeclose();
 extern vm_offset_t timemmap();
 #define	timename		"time"
 
+#ifndef	MACH_HYP
 extern int	kdopen(), kdclose(), kdread(), kdwrite();
 extern int	kdgetstat(), kdsetstat(), kdportdeath();
 extern vm_offset_t kdmmap();
@@ -52,21 +51,29 @@ extern int	lpropen(), lprclose(), lprread(), lprwrite();
 extern int	lprgetstat(), lprsetstat(), lprportdeath();
 #define	lprname			"lpr"
 #endif	/* NLPR > 0 */
+#endif	/* MACH_HYP */
 
 extern int	kbdopen(), kbdclose(), kbdread();
 extern int	kbdgetstat(), kbdsetstat();
 #define	kbdname			"kbd"
 
+#ifndef	MACH_HYP
 extern int	mouseopen(), mouseclose(), mouseread(), mousegetstat();
 #define	mousename		"mouse"
 
-extern int	ioplopen(), ioplclose();
-extern vm_offset_t ioplmmap();
-#define	ioplname		"iopl"
+extern vm_offset_t memmmap();
+#define	memname			"mem"
+#endif	/* MACH_HYP */
 
 extern int	kmsgopen(), kmsgclose(), kmsgread(), kmsggetstat();
 #define kmsgname		"kmsg"
 
+#ifdef	MACH_HYP
+extern int	hypcnopen(), hypcnclose(), hypcnread(), hypcnwrite();
+extern int	hypcngetstat(), hypcnsetstat(), hypcnportdeath();
+#define hypcnname		"hyp"
+#endif	/* MACH_HYP */
+
 /*
  * List of devices - console must be at slot 0
  */
@@ -81,20 +88,23 @@ struct dev_ops	dev_name_list[] =
 	   cninit() we stick something appropriate here through the
 	   indirect list */
 	{ "cn",		nulldev,	nulldev,	nulldev,
-	  nulldev,	nulldev,	nulldev,	nulldev,
+	  nulldev,	nulldev,	nulldev,	nomap,
 	  nodev,	nulldev,	nulldev,	0,
 	  nodev },
 
+#ifndef	MACH_HYP
 	{ kdname,	kdopen,		kdclose,	kdread,
 	  kdwrite,	kdgetstat,	kdsetstat,	kdmmap,
 	  nodev,	nulldev,	kdportdeath,	0,
 	  nodev },
+#endif	/* MACH_HYP */
 
 	{ timename,	timeopen,	timeclose,	nulldev,
 	  nulldev,	nulldev,	nulldev,	timemmap,
 	  nodev,	nulldev,	nulldev,	0,
 	  nodev },
 
+#ifndef	MACH_HYP
 #if	NCOM > 0
 	{ comname,	comopen,	comclose,	comread,
 	  comwrite,	comgetstat,	comsetstat,	nomap,
@@ -119,10 +129,11 @@ struct dev_ops	dev_name_list[] =
 	  nodev,	nulldev,	nulldev,	0,
 	  nodev },
 
-	{ ioplname,	ioplopen,	ioplclose,	nodev,
-	  nodev,	nodev,		nodev,		ioplmmap,
+	{ memname,	nulldev,	nulldev,	nodev,
+	  nodev,	nodev,		nodev,		memmmap,
 	  nodev,	nulldev,	nulldev,	0,
 	  nodev },
+#endif	/* MACH_HYP */
 
 #ifdef	MACH_KMSG
         { kmsgname,     kmsgopen,       kmsgclose,       kmsgread,
@@ -131,6 +142,13 @@ struct dev_ops	dev_name_list[] =
           nodev },
 #endif
 
+#ifdef	MACH_HYP
+	{ hypcnname,	hypcnopen,	hypcnclose,	hypcnread,
+	  hypcnwrite,	hypcngetstat,	hypcnsetstat,	nomap,
+	  nodev,	nulldev,	hypcnportdeath,	0,
+	  nodev },
+#endif	/* MACH_HYP */
+
 };
 int	dev_name_count = sizeof(dev_name_list)/sizeof(dev_name_list[0]);
 
