@@ -48,12 +48,11 @@ WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 #include <sys/types.h>
 #include <sys/time.h>
-#include <kern/time_out.h>
+#include <kern/mach_clock.h>
 #include <i386/machspl.h>
 #include <i386/pio.h>
 #include <i386at/rtc.h>
 
-static unsigned char rtc[RTC_NREG];
 static int first_rtcopen_ever = 1;
 
 void
@@ -75,7 +74,7 @@ unsigned char	*regs;
 		first_rtcopen_ever = 0;
 	}
 	outb(RTC_ADDR, RTC_D);
-	if (inb(RTC_DATA) & RTC_VRT == 0) return(-1);
+	if ((inb(RTC_DATA) & RTC_VRT) == 0) return(-1);
 	outb(RTC_ADDR, RTC_A);
 	while (inb(RTC_DATA) & RTC_UIP)		/* busy wait */
 		outb(RTC_ADDR, RTC_A);
@@ -139,11 +138,7 @@ readtodc(tp)
 	int i, days = 0;
 	spl_t	ospl;
 
-#ifdef	MACH_KERNEL
 	ospl = splclock();
-#else	/* MACH_KERNEL */
-	ospl = spl5();
-#endif	/* MACH_KERNEL */
 	if (rtcget(&rtclk)) {
 		splx(ospl);
 		return(-1);
@@ -170,12 +165,6 @@ readtodc(tp)
 		days += yeartoday(i);
 	n += days * 3600 * 24;
 
-#ifdef	MACH_KERNEL
-#else	/* MACH_KERNEL */
-	n += tz.tz_minuteswest * 60;
-	if (tz.tz_dsttime)
-		n -= 3600;
-#endif	/* MACH_KERNEL */
 
 	*tp = n;
 
@@ -190,24 +179,14 @@ writetodc()
 	int diff, i, j;
 	spl_t	ospl;
 
-#ifdef	MACH_KERNEL
 	ospl = splclock();
-#else	/* MACH_KERNEL */
-	ospl = spl5();
-#endif	/* MACH_KERNEL */
 	if (rtcget(&rtclk)) {
 		splx(ospl);
 		return(-1);
 	}
 	splx(ospl);
 
-#ifdef	MACH_KERNEL
 	diff = 0;
-#else	/* MACH_KERNEL */
-	diff = tz.tz_minuteswest * 60;
-	if (tz.tz_dsttime)
-		diff -= 3600;
-#endif	/* MACH_KERNEL */
 	n = (time.tv_sec - diff) % (3600 * 24);   /* hrs+mins+secs */
 	rtclk.rtc_sec = dectohexdec(n%60);
 	n /= 60;
@@ -231,11 +210,7 @@ writetodc()
 
 	rtclk.rtc_dom = dectohexdec(++n);
 
-#ifdef	MACH_KERNEL
 	ospl = splclock();
-#else	/* MACH_KERNEL */
-	ospl = spl5();
-#endif	/* MACH_KERNEL */
 	rtcput(&rtclk);
 	splx(ospl);
 
