@@ -28,11 +28,9 @@
 
 #include <string.h>
 
-#include <mach_ipc_compat.h> /* XXX */
 #include <mach/kern_return.h>
 #include <mach/alert.h>
-#include <kern/mach_param.h> /* XXX INCALL_... */
-#include <kern/zalloc.h>
+#include <kern/slab.h>
 #include <kern/thread.h>
 #include <kern/task.h>
 #include <kern/debug.h>
@@ -48,7 +46,7 @@ static void special_handler(ReturnHandler *rh, struct Act *act);
 #endif
 
 #ifndef ACT_STATIC_KLUDGE
-static zone_t act_zone;
+static struct kmem_cache act_cache;
 #else
 static Act *act_freelist;
 static Act free_acts[ACT_STATIC_KLUDGE];
@@ -69,11 +67,8 @@ void
 global_act_init()
 {
 #ifndef ACT_STATIC_KLUDGE
-	act_zone = zinit(
-			sizeof(struct Act),
-			ACT_MAX * sizeof(struct Act), /* XXX */
-			ACT_CHUNK * sizeof(struct Act),
-			0, "activations");
+	kmem_cache_init(&act_cache, "Act", sizeof(struct Act), 0,
+			NULL, NULL, NULL, 0);
 #else
 	int i;
 
@@ -102,10 +97,9 @@ kern_return_t act_create(task_t task, vm_offset_t user_stack,
 			 struct Act **new_act)
 {
 	Act *act;
-	int rc;
 
 #ifndef ACT_STATIC_KLUDGE
-	act = (Act*)zalloc(act_zone);
+	act = (Act*)kmem_cache_alloc(&act_cache);
 	if (act == 0)
 		return(KERN_RESOURCE_SHORTAGE);
 #else
@@ -171,9 +165,9 @@ static void act_free(Act *inc)
 	/* Drop the task reference.  */
 	task_deallocate(inc->task);
 
-	/* Put the act back on the act zone */
+	/* Put the act back on the act cache */
 #ifndef ACT_STATIC_KLUDGE
-	zfree(act_zone, (vm_offset_t)inc);
+	kmem_cache_free(&act_cache, (vm_offset_t)inc);
 #else
 	/* XXX ipt_lock(act_freelist); */
 	inc->ipt_next = act_freelist;
@@ -933,12 +927,6 @@ act_get_special_port(Act *act, int which, ipc_port_t *portp)
 		return KERN_INVALID_ARGUMENT;
 
 	switch (which) {
-#if	MACH_IPC_COMPAT
-	    case THREAD_REPLY_PORT:
-		whichp = &act->reply_port;
-		break;
-#endif	/* MACH_IPC_COMPAT */
-
 	    case THREAD_KERNEL_PORT:
 		whichp = &act->self_port;
 		break;
@@ -993,12 +981,6 @@ act_set_special_port(Act *act, int which, ipc_port_t port)
 		return KERN_INVALID_ARGUMENT;
 
 	switch (which) {
-#if	MACH_IPC_COMPAT
-	    case THREAD_REPLY_PORT:
-		whichp = &act->reply_port;
-		break;
-#endif	/* MACH_IPC_COMPAT */
-
 	    case THREAD_KERNEL_PORT:
 		whichp = &act->self_port;
 		break;
