@@ -31,12 +31,12 @@
 
 #include <kern/macro_help.h>
 
-#ifdef DEBUG
+#ifndef NDEBUG
 #define MACH_ASSERT 1
 #endif
 
 #if	MACH_ASSERT
-extern void Assert(char *exp, char *filename, int line);
+extern void Assert(char *exp, char *filename, int line) __attribute__ ((noreturn));
 
 #define assert(ex)							\
 MACRO_BEGIN								\
