@@ -29,10 +29,10 @@
 #include "cpu_number.h"
 #include <kern/debug.h>
 #include <kern/lock.h>
+#include <kern/mach_clock.h>
 #include <kern/thread.h>
 #include <kern/sched_prim.h>
 #include <kern/processor.h>
-#include <kern/time_out.h>
 #include <kern/thread_swap.h>
 #include <kern/ipc_sched.h>
 #include <machine/machspl.h>	/* for splsched/splx */
@@ -166,7 +166,6 @@ thread_will_wait_with_timeout(
 #define	check_bound_processor(thread)	TRUE
 #endif	/* NCPUS > 1 */
 
-#ifdef CONTINUATIONS
 /*
  *	Routine:	thread_handoff
  *	Purpose:
@@ -282,4 +281,3 @@ thread_handoff(
 	counter_always(c_thread_handoff_hits++);
 	return TRUE;
 }
-#endif /* CONTINUATIONS */
