@@ -30,9 +30,12 @@
 #include <mach/mach_types.h>	/* vm_address_t */
 #include <mach/std_types.h>	/* pointer_t */
 #include <mach/pc_sample.h>
+#include <machine/trap.h>
+#include <kern/kalloc.h>
 #include <kern/host.h>
 #include <kern/thread.h>
 #include <kern/pc_sample.h>
+#include <kern/mach_clock.h>
 
 #if	MACH_PCSAMPLE
 
@@ -54,7 +57,7 @@ void take_pc_sample(
     pc = interrupted_pc(t);
     cp->seqno++;
     sample = &((sampled_pc_t *)cp->buffer)[cp->seqno % MAX_PC_SAMPLES];
-    sample->id = (natural_t)t;
+    sample->id = (vm_offset_t)t;
     sample->pc = pc;
     sample->sampletype = flavor;
 }
@@ -66,7 +69,6 @@ thread_enable_pc_sampling(
     sampled_pc_flavor_t flavors)
 {
     vm_offset_t buf;
-    extern int tick;
 
     if (thread == THREAD_NULL)  {
 	return KERN_INVALID_ARGUMENT;
@@ -92,7 +94,6 @@ task_enable_pc_sampling(
     sampled_pc_flavor_t flavors)
 {
     vm_offset_t buf;
-    extern int tick;
 
     if (task == TASK_NULL)  {
 	return KERN_INVALID_ARGUMENT;
