@@ -40,10 +40,9 @@
 #include <ipc/ipc_space.h>
 #include <ipc/ipc_types.h>
 #include <kern/debug.h>
-#include <kern/mach_param.h>
 #include <kern/task.h>
 #include <kern/thread.h>
-#include <kern/zalloc.h>
+#include <kern/slab.h>
 #include <kern/kalloc.h>
 #include <kern/processor.h>
 #include <kern/sched_prim.h>	/* for thread_wakeup */
@@ -52,7 +51,7 @@
 #include <machine/machspl.h>	/* for splsched */
 
 task_t	kernel_task = TASK_NULL;
-zone_t	task_zone;
+struct kmem_cache task_cache;
 
 extern void eml_init(void);
 extern void eml_task_reference(task_t, task_t);
@@ -60,13 +59,11 @@ extern void eml_task_deallocate(task_t);
 
 void task_init(void)
 {
-	task_zone = zinit(
-			sizeof(struct task),
-			TASK_MAX * sizeof(struct task),
-			TASK_CHUNK * sizeof(struct task),
-			0, "tasks");
+	kmem_cache_init(&task_cache, "task", sizeof(struct task), 0,
+			NULL, NULL, NULL, 0);
 
 	eml_init();
+	machine_task_module_init ();
 
 	/*
 	 * Create the kernel task as the first task.
@@ -76,38 +73,6 @@ void task_init(void)
 	(void) task_create(TASK_NULL, FALSE, &kernel_task);
 }
 
-/*
- * Create a task running in the kernel address space.  It may
- * have its own map of size mem_size (if 0, it uses the kernel map),
- * and may have ipc privileges.
- */
-task_t	kernel_task_create(
-	task_t		parent_task,
-	vm_size_t	map_size)
-{
-	task_t		new_task;
-	vm_offset_t	min, max;
-
-	/*
-	 * Create the task.
-	 */
-	(void) task_create(parent_task, FALSE, &new_task);
-
-	/*
-	 * Task_create creates the task with a user-space map.
-	 * Remove the map and replace it with the kernel map
-	 * or a submap of the kernel map.
-	 */
-	vm_map_deallocate(new_task->map);
-	if (map_size == 0)
-	    new_task->map = kernel_map;
-	else
-	    new_task->map = kmem_suballoc(kernel_map, &min, &max,
-					  map_size, FALSE);
-
-	return new_task;
-}
-
 kern_return_t task_create(
 	task_t		parent_task,
 	boolean_t	inherit_memory,
@@ -115,9 +80,11 @@ kern_return_t task_create(
 {
 	register task_t	new_task;
 	register processor_set_t	pset;
+#if FAST_TAS
 	int i;
+#endif
 
-	new_task = (task_t) zalloc(task_zone);
+	new_task = (task_t) kmem_cache_alloc(&task_cache);
 	if (new_task == TASK_NULL) {
 		panic("task_create: no memory for task structure");
 	}
@@ -141,10 +108,18 @@ kern_return_t task_create(
 	new_task->active = TRUE;
 	new_task->user_stop_count = 0;
 	new_task->thread_count = 0;
+	new_task->faults = 0;
+	new_task->zero_fills = 0;
+	new_task->reactivations = 0;
+	new_task->pageins = 0;
+	new_task->cow_faults = 0;
+	new_task->messages_sent = 0;
+	new_task->messages_received = 0;
 
 	eml_task_reference(new_task, parent_task);
 
 	ipc_task_init(new_task, parent_task);
+	machine_task_init (new_task);
 
 	new_task->total_user_time.seconds = 0;
 	new_task->total_user_time.microseconds = 0;
@@ -220,6 +195,8 @@ void task_deallocate(
 	if (c != 0)
 		return;
 
+	machine_task_terminate (task);
+
 	eml_task_deallocate(task);
 
 	pset = task->processor_set;
@@ -229,7 +206,7 @@ void task_deallocate(
 	pset_deallocate(pset);
 	vm_map_deallocate(task->map);
 	is_release(task->itk_space);
-	zfree(task_zone, (vm_offset_t) task);
+	kmem_cache_free(&task_cache, (vm_offset_t) task);
 }
 
 void task_reference(
@@ -777,6 +754,30 @@ kern_return_t task_info(
 		break;
 	    }
 
+	    case TASK_EVENTS_INFO:
+	    {
+		register task_events_info_t	event_info;
+
+		if (*task_info_count < TASK_EVENTS_INFO_COUNT) {
+		    return KERN_INVALID_ARGUMENT;
+		}
+
+		event_info = (task_events_info_t) task_info_out;
+
+		task_lock(&task);
+		event_info->faults = task->faults;
+		event_info->zero_fills = task->zero_fills;
+		event_info->reactivations = task->reactivations;
+		event_info->pageins = task->pageins;
+		event_info->cow_faults = task->cow_faults;
+		event_info->messages_sent = task->messages_sent;
+		event_info->messages_received = task->messages_received;
+		task_unlock(&task);
+
+		*task_info_count = TASK_EVENTS_INFO_COUNT;
+		break;
+	    }
+
 	    case TASK_THREAD_TIMES_INFO:
 	    {
 		register task_thread_times_info_t times_info;
@@ -1092,6 +1093,7 @@ void task_collect_scan(void)
 			pset_unlock(pset);
 			simple_unlock(&all_psets_lock);
 
+			machine_task_collect (task);
 			pmap_collect(task->map->pmap);
 
 			if (prev_task != TASK_NULL)
